@@ -2,7 +2,7 @@
 #include "exportFile.h"
 using namespace Rcpp;
 using namespace std;
-
+using namespace arma;
 
 /********************************************
 *  Constructors ~ Model initialization      *
@@ -11,6 +11,7 @@ using namespace std;
 //  Initialization of a hidden Markov model with categorical observations
 //  It has n - hidden states
 //  It has m - possible emissions
+// [[Rcpp::depends(RcppArmadillo)]]
 // [[Rcpp::export]]
 SEXP initHMM(SEXP n, SEXP m)
 {
@@ -31,14 +32,15 @@ SEXP initHMM(SEXP n, SEXP m)
 }
 
 //  Initialization of a hidden Markov model with continuous observations
-//  It has n - hidden states
+//  It has n - hidden states and m - dimensions
+// [[Rcpp::depends(RcppArmadillo)]] 
 // [[Rcpp::export]]
-SEXP initGHMM(SEXP n)
+SEXP initGHMM(SEXP n, SEXP m)
 {
     //  error handling
     try {
         // Model initialization
-        HMMcont hmm(as<unsigned int>(n));            
+        MultiGHMM hmm(as<unsigned int>(n), as<unsigned int>(m));            
         //  Return Model parameters to R environment as a list instead of an object  
         return hmm.toList();       
     } catch(std::exception &ex) {	
@@ -52,6 +54,7 @@ SEXP initGHMM(SEXP n)
 
 //  Initialization of a hidden Markov model with categorical observations
 //  It has n - hidden states
+// [[Rcpp::depends(RcppArmadillo)]] 
 // [[Rcpp::export]]
 SEXP initPHMM(SEXP n)
 {
@@ -71,6 +74,7 @@ SEXP initPHMM(SEXP n)
 }
 
 //  Function used to verify that all the parameters fulfill the model requirements
+// [[Rcpp::depends(RcppArmadillo)]]
 // [[Rcpp::export]]
 SEXP verifyModel(SEXP model)
 {       
@@ -87,19 +91,19 @@ SEXP verifyModel(SEXP model)
             HMM newHMM( init_hmm["StateNames"] , init_hmm["ObservationNames"], init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
             //  Return Model parameters to R environment as a list instead of an object
             return newHMM.toList();
-        }        
-        else if(model == "GHMM")
-        {
-            HMMcont newHMM( init_hmm["StateNames"] , init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
-            //  Return Model parameters to R environment as a list instead of an object
-            return newHMM.toList();
-        }        
+        }                
         else if(model == "PHMM")
         {
             HMMpoisson newHMM( init_hmm["StateNames"] , init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
             //  Return Model parameters to R environment as a list instead of an object
             return newHMM.toList();
-        }        
+        }    
+        else if(model == "GHMM")   
+        {
+            MultiGHMM newHMM( init_hmm["StateNames"] , as<mat>(init_hmm["A"]), as<mat>(init_hmm["Mu"]),as<cube>(init_hmm["Sigma"]), as<rowvec>(init_hmm["Pi"]));
+            //  Return Model parameters to R environment as a list instead of an object
+            return newHMM.toList();
+        }     
         else
             Rf_error("That model is not supported.");
 
@@ -117,6 +121,7 @@ SEXP verifyModel(SEXP model)
 ********************************************/
 
 //  Set names: State names and Emission names
+// [[Rcpp::depends(RcppArmadillo)]]
 // [[Rcpp::export]]
 SEXP setNames(SEXP hmm, SEXP names)
 {
@@ -134,17 +139,17 @@ SEXP setNames(SEXP hmm, SEXP names)
             //  Return Model parameters to R environment as a list instead of an object
             return newHMM.toList();
         }        
-        else if(model == "GHMM")
-        {
-            HMMcont newHMM( init_names["StateNames"] , init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
-            //  Return Model parameters to R environment as a list instead of an object
-            return newHMM.toList();
-        }            
         else if(model == "PHMM")   
         {
             HMMpoisson newHMM( init_names["StateNames"] , init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
             //  Return Model parameters to R environment as a list instead of an object
             return newHMM.toList();
+        }
+        else if(model == "GHMM")   
+        {
+            MultiGHMM newHMM( init_names["StateNames"] , as<mat>(init_hmm["A"]), as<mat>(init_hmm["Mu"]),as<cube>(init_hmm["Sigma"]), as<rowvec>(init_hmm["Pi"]));
+            //  Return Model parameters to R environment as a list instead of an object
+            return newHMM.toList();
         }                 
         else
             Rf_error("That model is not supported.");
@@ -159,6 +164,7 @@ SEXP setNames(SEXP hmm, SEXP names)
 }
 
 //  Set parameters: A: Transition matrix. B: Emission matrix. Pi: Initial probability vector
+// [[Rcpp::depends(RcppArmadillo)]]
 // [[Rcpp::export]]
 SEXP setParameters(SEXP hmm, SEXP params)
 {  
@@ -175,19 +181,19 @@ SEXP setParameters(SEXP hmm, SEXP params)
             HMM newHMM( init_hmm["StateNames"] , init_hmm["ObservationNames"], init_params["A"], init_params["B"], init_params["Pi"]);
             //  Return Model parameters to R environment as a list instead of an object
             return newHMM.toList();
-        }                         
-        else if(model == "GHMM")
-        {
-            HMMcont newHMM( init_hmm["StateNames"] , init_params["A"], init_params["B"], init_params["Pi"]);
-            //  Return Model parameters to R environment as a list instead of an object
-            return newHMM.toList();
-        }                 
+        }                                
         else if(model == "PHMM")
         {
             HMMpoisson newHMM( init_hmm["StateNames"] , init_params["A"], init_params["B"], init_params["Pi"]);
             //  Return Model parameters to R environment as a list instead of an object
             return newHMM.toList();
-        }            
+        } 
+        else if(model == "GHMM")   
+        {
+            MultiGHMM newHMM( init_hmm["StateNames"] , as<mat>(init_params["A"]), as<mat>(init_params["Mu"]), as<cube>(init_params["Sigma"]), as<rowvec>(init_params["Pi"]));
+            //  Return Model parameters to R environment as a list instead of an object
+            return newHMM.toList();
+        }           
         else
             Rf_error("That model is not supported.");
                                
@@ -204,6 +210,7 @@ SEXP setParameters(SEXP hmm, SEXP params)
 *           Sequence Evaluation             *
 ********************************************/
 
+// [[Rcpp::depends(RcppArmadillo)]]
 // [[Rcpp::export]]
 SEXP evaluation(SEXP hmm, SEXP sequence, SEXP method)
 {    
@@ -219,16 +226,19 @@ SEXP evaluation(SEXP hmm, SEXP sequence, SEXP method)
         {
             HMM newHMM( init_hmm["StateNames"] , init_hmm["ObservationNames"], init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
             return wrap(newHMM.evaluation(sequence, init_method));
-        }else if(model == "GHMM")
-        {
-            HMMcont newHMM( init_hmm["StateNames"] , init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
-            return wrap(newHMM.evaluation(sequence, init_method));
         }
         else if(model == "PHMM")
         {
             HMMpoisson newHMM( init_hmm["StateNames"] , init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
             return wrap(newHMM.evaluation(sequence, init_method));            
-        }else
+        }
+        else if(model == "GHMM")   
+        {
+            //  We init the model
+            MultiGHMM newHMM( init_hmm["StateNames"] , as<mat>(init_hmm["A"]), as<mat>(init_hmm["Mu"]),as<cube>(init_hmm["Sigma"]), as<rowvec>(init_hmm["Pi"]));
+            return wrap(newHMM.evaluation(as<mat>(sequence), init_method));
+        }
+        else
             Rf_error("That model is not supported.");
         
     } catch(std::exception &ex) {	
@@ -244,6 +254,7 @@ SEXP evaluation(SEXP hmm, SEXP sequence, SEXP method)
 *          Sequence Decodification          *
 ********************************************/
 
+// [[Rcpp::depends(RcppArmadillo)]]
 // [[Rcpp::export]]
 SEXP viterbi(SEXP hmm, SEXP sequence)
 {    
@@ -256,15 +267,18 @@ SEXP viterbi(SEXP hmm, SEXP sequence)
         {
             HMM newHMM( init_hmm["StateNames"] , init_hmm["ObservationNames"], init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
             return newHMM.viterbi(sequence);
-        }else if(model == "GHMM")
-        {
-            HMMcont newHMM( init_hmm["StateNames"] , init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
-            return newHMM.viterbi(sequence);
         }else if(model == "PHMM")
         {
             HMMpoisson newHMM( init_hmm["StateNames"] , init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
             return newHMM.viterbi(sequence);
-        }else
+        }
+        else if(model == "GHMM")   
+        {
+            //  We init the model
+            MultiGHMM newHMM( init_hmm["StateNames"] , as<mat>(init_hmm["A"]), as<mat>(init_hmm["Mu"]),as<cube>(init_hmm["Sigma"]), as<rowvec>(init_hmm["Pi"]));
+            return newHMM.viterbi(as<mat>(sequence));
+        }
+        else
             Rf_error("That model is not supported.");
 
     } catch(std::exception &ex) {	
@@ -276,6 +290,7 @@ SEXP viterbi(SEXP hmm, SEXP sequence)
     return wrap(NA_REAL);     
 }
 
+// [[Rcpp::depends(RcppArmadillo)]]
 // [[Rcpp::export]]
 SEXP forwardBackward(SEXP hmm, SEXP sequence)
 {    
@@ -289,15 +304,18 @@ SEXP forwardBackward(SEXP hmm, SEXP sequence)
         {
             HMM newHMM( init_hmm["StateNames"] , init_hmm["ObservationNames"], init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
             return newHMM.forwardBackward(sequence);
-        }else if(model == "GHMM")
-        {
-            HMMcont newHMM( init_hmm["StateNames"] , init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
-            return newHMM.forwardBackward(sequence);
         }else if(model == "PHMM")
         {
             HMMpoisson newHMM( init_hmm["StateNames"] , init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
             return newHMM.forwardBackward(sequence);
-        }else
+        }
+        else if(model == "GHMM")   
+        {
+            //  We init the model
+            MultiGHMM newHMM( init_hmm["StateNames"] , as<mat>(init_hmm["A"]), as<mat>(init_hmm["Mu"]),as<cube>(init_hmm["Sigma"]), as<rowvec>(init_hmm["Pi"]));
+            return newHMM.forwardBackward(as<mat>(sequence) );
+        }
+        else
             Rf_error("That model is not supported.");
 
     } catch(std::exception &ex) {	
@@ -314,6 +332,7 @@ SEXP forwardBackward(SEXP hmm, SEXP sequence)
 ********************************************/
 
 //  Function used to determine the loglikelihood of a sequence set given a specific model
+// [[Rcpp::depends(RcppArmadillo)]]
 // [[Rcpp::export]]
 SEXP loglikelihood(SEXP hmm, SEXP sequences)
 {    
@@ -327,15 +346,17 @@ SEXP loglikelihood(SEXP hmm, SEXP sequences)
         {
             HMM newHMM( init_hmm["StateNames"] , init_hmm["ObservationNames"], init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
             return wrap(newHMM.loglikelihood(sequences));
-        }else if(model == "GHMM")
-        {
-            HMMcont newHMM( init_hmm["StateNames"] , init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
-            return wrap(newHMM.loglikelihood(sequences));
         }else if(model == "PHMM")
         {
             HMMpoisson newHMM( init_hmm["StateNames"] , init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
             return wrap(newHMM.loglikelihood(sequences));
-        }else
+        }
+        else if(model == "GHMM")   
+        {
+            MultiGHMM newHMM( init_hmm["StateNames"] , as<mat>(init_hmm["A"]), as<mat>(init_hmm["Mu"]),as<cube>(init_hmm["Sigma"]), as<rowvec>(init_hmm["Pi"]));
+            return wrap(newHMM.loglikelihood(as<cube>(sequences)) );
+        }
+        else
             Rf_error("That model is not supported.");
 
     } catch(std::exception &ex) {	
@@ -348,6 +369,7 @@ SEXP loglikelihood(SEXP hmm, SEXP sequences)
 }
 
 //  Function used for parameter estimation given two or more observation sequences
+// [[Rcpp::depends(RcppArmadillo)]]
 // [[Rcpp::export]] 
 SEXP learnEM(SEXP hmm, SEXP sequences, SEXP iter, SEXP delta, SEXP pseudo, SEXP print)
 {
@@ -371,17 +393,18 @@ SEXP learnEM(SEXP hmm, SEXP sequences, SEXP iter, SEXP delta, SEXP pseudo, SEXP
             HMM newHMM( init_hmm["StateNames"] , init_hmm["ObservationNames"], init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);        
             newHMM.learnEM(sequences, init_iter, init_delta, init_pseudo, init_print);
             return newHMM.toList();
-        }else if(model == "GHMM")
-        {                      
-            HMMcont newHMM( init_hmm["StateNames"] , init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
-            newHMM.learnEM(sequences, init_iter, init_delta, init_pseudo, init_print);
-            return newHMM.toList();           
-        } else if(model == "PHMM")
+        }else if(model == "PHMM")
         {                      
             HMMpoisson newHMM( init_hmm["StateNames"] , init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
             newHMM.learnEM(sequences, init_iter, init_delta, init_pseudo, init_print);
             return newHMM.toList();        
-        }else
+        }else if(model == "GHMM")   
+        {
+            MultiGHMM newHMM( init_hmm["StateNames"] , as<mat>(init_hmm["A"]), as<mat>(init_hmm["Mu"]),as<cube>(init_hmm["Sigma"]), as<rowvec>(init_hmm["Pi"]));
+            newHMM.learnEM(as<cube>(sequences), init_iter, init_delta, init_pseudo, init_print);
+            return newHMM.toList(); 
+        }
+        else
             Rf_error("That model is not supported.");
          
     } catch(std::exception &ex) {	
@@ -394,6 +417,7 @@ SEXP learnEM(SEXP hmm, SEXP sequences, SEXP iter, SEXP delta, SEXP pseudo, SEXP
 }
 
 //  Function used for parameter estimation given only one observation sequence
+// [[Rcpp::depends(RcppArmadillo)]]
 // [[Rcpp::export]]
 SEXP learnBW(SEXP hmm, SEXP sequences, SEXP iter, SEXP delta, SEXP pseudo, SEXP print)
 {
@@ -417,16 +441,16 @@ SEXP learnBW(SEXP hmm, SEXP sequences, SEXP iter, SEXP delta, SEXP pseudo, SEXP
             HMM newHMM( init_hmm["StateNames"] , init_hmm["ObservationNames"], init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);        
             newHMM.learnBW(sequences, init_iter, init_delta, init_pseudo, init_print);
             return newHMM.toList();
-        }else if(model == "GHMM")
-        {                      
-            HMMcont newHMM( init_hmm["StateNames"] , init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
-            newHMM.learnBW(sequences, init_iter, init_delta, init_pseudo, init_print);
-            return newHMM.toList();          
         }else if(model == "PHMM")
         {                      
             HMMpoisson newHMM( init_hmm["StateNames"] , init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
             newHMM.learnBW(sequences, init_iter, init_delta, init_pseudo, init_print);
             return newHMM.toList();         
+        }else if(model == "GHMM")   
+        {
+            MultiGHMM newHMM( init_hmm["StateNames"] , as<mat>(init_hmm["A"]), as<mat>(init_hmm["Mu"]),as<cube>(init_hmm["Sigma"]), as<rowvec>(init_hmm["Pi"]));
+            newHMM.learnBW(as<mat>(sequences), init_iter, init_delta, init_pseudo, init_print);
+            return newHMM.toList(); 
         }else
             Rf_error("That model is not supported.");
          
@@ -445,6 +469,7 @@ SEXP learnBW(SEXP hmm, SEXP sequences, SEXP iter, SEXP delta, SEXP pseudo, SEXP
 ********************************************/
 
 // Given a specific model, it generates an observation sequence and the hidden states that originated them.
+// [[Rcpp::depends(RcppArmadillo)]]
 // [[Rcpp::export]] 
 SEXP generateObservations(SEXP hmm, SEXP length)
 {
@@ -461,14 +486,14 @@ SEXP generateObservations(SEXP hmm, SEXP length)
         {
             HMM newHMM( init_hmm["StateNames"] , init_hmm["ObservationNames"], init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
             return newHMM.generateObservations(init_length);
-        }else if(model == "GHMM")
-        {
-            HMMcont newHMM( init_hmm["StateNames"] , init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
-            return newHMM.generateObservations(init_length);
         }else if(model == "PHMM")
         {
             HMMpoisson newHMM( init_hmm["StateNames"] , init_hmm["A"], init_hmm["B"], init_hmm["Pi"]);
             return newHMM.generateObservations(init_length);
+        }else if(model == "GHMM")   
+        {
+            MultiGHMM newHMM( init_hmm["StateNames"] , as<mat>(init_hmm["A"]), as<mat>(init_hmm["Mu"]),as<cube>(init_hmm["Sigma"]), as<rowvec>(init_hmm["Pi"]));
+            return newHMM.generateObservations(init_length);
         }else
             Rf_error("That model is not supported.");
     } catch(std::exception &ex) {	
